@@ -1,6 +1,6 @@
 /*
  * Copyright 2003 by John Joganic <john@joganic.com>
- * Copyright 2003 - 2008 by Ping Cheng <pingc@wacom.com> 
+ * Copyright 2003 - 2009 by Ping Cheng <pingc@wacom.com> 
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -39,6 +39,7 @@
 #define XWACOM_PARAM_COMMONDBG		15
 #define XWACOM_PARAM_RAWSAMPLE		16
 #define XWACOM_PARAM_CAPACITY       	17
+#define XWACOM_PARAM_SERIAL		18
 
 /* The followings are defined together */
 #define XWACOM_PARAM_BUTTON1    	101
@@ -95,11 +96,11 @@
 #define XWACOM_PARAM_TVRESOLUTION0	207
 #define XWACOM_PARAM_TVRESOLUTION1	208
 #define XWACOM_PARAM_COREEVENT		209
+#define XWACOM_PARAM_THRESHOLD	 	210
 
 #define XWACOM_PARAM_GETONLYPARAM	320
 #define XWACOM_PARAM_TID		321
 #define XWACOM_PARAM_TOOLID		322
-#define XWACOM_PARAM_TOOLSERIAL		323
 #define XWACOM_PARAM_NUMSCREEN		350
 #define XWACOM_PARAM_STOPX0		351
 #define XWACOM_PARAM_STOPY0		352
@@ -134,10 +135,15 @@
 #define XWACOM_PARAM_SBOTTOMX7		381
 #define XWACOM_PARAM_SBOTTOMY7		382
 #define XWACOM_PARAM_XSCALING		383
+#define XWACOM_PARAM_TOOLSERIAL		384
 
 #define TV_NONE 		0
-#define TV_ABOVE_BELOW 		1
-#define TV_LEFT_RIGHT		2
+#define TV_XINERAMA		1
+#define TV_ABOVE_BELOW 		2
+#define TV_LEFT_RIGHT		3
+#define TV_BELOW_ABOVE		4
+#define TV_RIGHT_LEFT		5
+#define TV_MAX			5
 
 #define ROTATE_NONE 		0
 #define ROTATE_CW 		1
@@ -155,7 +161,8 @@
 #define AC_KEY              0x00010000	/* Emit key events */
 #define AC_MODETOGGLE       0x00020000	/* Toggle absolute/relative mode */
 #define AC_DBLCLICK         0x00030000	/* Emit a button1 double-click event */
-#define AC_DISPLAYTOGGLE    0x00040000 /* Toggle among screens */
+#define AC_DISPLAYTOGGLE    0x00040000  /* Toggle among displays (screen plus whole desktop) */
+#define AC_SCREENTOGGLE     0x00050000  /* Toggle among screens */
 #define AC_TYPE             0x000f0000	/* The mask to isolate event type bits */
 #define AC_NUM_KEYS         0x0ff00000  /* The mask to isolate number of keys to send */
 #define AC_CORE             0x10000000	/* Always emit a core event */
