@@ -1,6 +1,6 @@
 /*
  * Copyright 1995-2002 by Frederic Lepied, France. <Lepied@XFree86.org>
- * Copyright 2002-2008 by Ping Cheng, Wacom Technology. <pingc@wacom.com>		
+ * Copyright 2002-2009 by Ping Cheng, Wacom Technology. <pingc@wacom.com>		
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -45,6 +45,9 @@ WacomDeviceClass* wcmDeviceClasses[] =
 extern int xf86WcmDevSwitchModeCall(LocalDevicePtr local, int mode);
 extern void xf86WcmChangeScreen(LocalDevicePtr local, int value);
 extern void xf86WcmInitialCoordinates(LocalDevicePtr local, int axes);
+extern void xf86WcmVirtaulTabletSize(LocalDevicePtr local);
+extern void xf86WcmVirtaulTabletPadding(LocalDevicePtr local);
+extern void xf86WcmTilt2R(WacomDeviceStatePtr ds);
 
 /*****************************************************************************
  * Static functions
@@ -56,7 +59,7 @@ static void commonDispatchDevice(WacomCommonPtr common, unsigned int channel,
 static void resetSampleCounter(const WacomChannelPtr pChannel);
 static void sendAButton(LocalDevicePtr local, int button, int mask,
 		int rx, int ry, int rz, int v3, int v4, int v5);
- 
+
 /*****************************************************************************
  * xf86WcmMappingFactor --
  *   calculate the proper tablet to screen mapping factor according to the 
@@ -66,13 +69,10 @@ static void sendAButton(LocalDevicePtr local, int button, int mask,
 void xf86WcmMappingFactor(LocalDevicePtr local)
 {
 	WacomDevicePtr priv = (WacomDevicePtr) local->private;
-	int i = 0, minX = 0, minY = 0, maxX = 0, maxY = 0;
  
 	DBG(10, priv->debugLevel, ErrorF("xf86WcmMappingFactor \n"));
 
-	priv->sizeX = priv->bottomX - priv->topX - 2*priv->tvoffsetX;
-	priv->sizeY = priv->bottomY - priv->topY - 2*priv->tvoffsetY;
-	priv->maxWidth = 0, priv->maxHeight = 0;
+	xf86WcmVirtaulTabletSize(local);
 	
 	if (!(priv->flags & ABSOLUTE_FLAG) || !priv->wcmMMonitor)
 	{
@@ -104,44 +104,11 @@ void xf86WcmMappingFactor(LocalDevicePtr local)
 	if (priv->currentScreen == -1) /* tool on the tablet */
 		priv->currentScreen = 0;
 
-	if ( ((priv->twinview != TV_NONE) || /* TwinView & whole desktop */
-		/* stay in one screen at a time (multimonitor) */
-		!priv->wcmMMonitor || 
-		/* always stay in the configured screen */
-		(screenInfo.numScreens > 1 && priv->screen_no != -1))  
-		 && (priv->flags & ABSOLUTE_FLAG) )
-	{
-		priv->maxWidth = priv->screenBottomX[priv->currentScreen] - 
-			priv->screenTopX[priv->currentScreen];
-		priv->maxHeight = priv->screenBottomY[priv->currentScreen] - 
-			priv->screenTopY[priv->currentScreen];
-	}
-	else
-	{
-		/* count the whole desktop when no specific screen is defined or 
-		 * tool is in relative mode
-		 */
-		minX = priv->screenTopX[0];
-		minY = priv->screenTopY[0];
-		maxX = priv->screenBottomX[0];
-		maxY = priv->screenBottomY[0];
-		for (i = 1; i < priv->numScreen; i++)
-		{
-			if (priv->screenTopX[i] < minX)
-				minX = priv->screenTopX[i];
-			if (priv->screenTopY[i] < minY)
-				minY = priv->screenTopY[i];
-			if (priv->screenBottomX[i] > maxX)
-				maxX = priv->screenBottomX[i];
-			if (priv->screenBottomY[i] > maxY)
-				maxY = priv->screenBottomY[i];
-		}
-		priv->maxWidth = maxX - minX;
-		priv->maxHeight = maxY - minY;
-	}
 	DBG(10, priv->debugLevel, ErrorF("xf86WcmMappingFactor"
-		" Active tablet area x=%d y=%d map to maxWidth =%d maxHeight =%d\n", 
-		priv->sizeX, priv->sizeY, priv->maxWidth, priv->maxHeight));
+		" Active tablet area x=%d y=%d (virtual tablet area x=%d y=%d) map"
+		" to maxWidth =%d maxHeight =%d\n", 
+		priv->bottomX, priv->bottomY, priv->sizeX, priv->sizeY, 
+		priv->maxWidth, priv->maxHeight));
 
 	priv->factorX = (double)priv->maxWidth / (double)priv->sizeX;
 	priv->factorY = (double)priv->maxHeight / (double)priv->sizeY;
@@ -157,34 +124,59 @@ void xf86WcmMappingFactor(LocalDevicePtr local)
  *   combined horizontal and vertical setups
  ****************************************************************************/
 
-static void xf86WcmSetScreen(LocalDevicePtr local, int *value0, int *value1)
+static void xf86WcmSetScreen(LocalDevicePtr local, int v0, int v1)
 {
 	WacomDevicePtr priv = (WacomDevicePtr) local->private;
-	int screenToSet = -1, letfPadding = 0, topPadding = 0;
-	int i, j, x, y, v0 = *value0, v1 = *value1;
+	int screenToSet = -1, i, j, x, y, tabletSize = 0;
 
-	DBG(6, priv->debugLevel, ErrorF("xf86WcmSetScreen "
-		"v0=%d v1=%d currentScreen=%d\n", *value0, 
-		*value1, priv->currentScreen));
+	DBG(6, priv->debugLevel, ErrorF("xf86WcmSetScreen v0=%d v1=%d "
+		"currentScreen=%d\n", v0, v1, priv->currentScreen));
+
+	if (priv->screen_no != -1 && priv->screen_no >= priv->numScreen)
+	{
+		ErrorF("xf86WcmSetScreen Screen%d is larger than number of available screens (%d)\n", 
+			priv->screen_no, priv->numScreen);
+		priv->screen_no = -1;
+	}
 
 	if (!(local->flags & (XI86_ALWAYS_CORE | XI86_CORE_POINTER))) return;
 
-	if (priv->twinview != TV_NONE && priv->screen_no == -1 && (priv->flags & ABSOLUTE_FLAG))
+	if (priv->twinview > TV_XINERAMA && priv->screen_no == -1 && (priv->flags & ABSOLUTE_FLAG))
 	{
 		if (priv->twinview == TV_LEFT_RIGHT)
 		{
-			if (v0 > priv->bottomX - priv->tvoffsetX && v0 <= priv->bottomX)
+			tabletSize = priv->bottomX - priv->tvoffsetX;
+			if (v0 > tabletSize && v0 <= priv->bottomX)
 				priv->currentScreen = 1;
 			if (v0 > priv->topX && v0 <= priv->topX + priv->tvoffsetX)
 				priv->currentScreen = 0;
 		}
 		if (priv->twinview == TV_ABOVE_BELOW)
 		{
-			if (v1 > priv->bottomY - priv->tvoffsetY && v1 <= priv->bottomY)
+			tabletSize = priv->bottomY - priv->tvoffsetY;
+			if (v0 > tabletSize && v0 <= priv->bottomY)
 				priv->currentScreen = 1;
-			if (v1 > priv->topY && v1 <= priv->topY + priv->tvoffsetY)
+			if (v0 > priv->topY && v0 <= priv->topY + priv->tvoffsetY)
 				priv->currentScreen = 0;
 		}
+		if (priv->twinview == TV_RIGHT_LEFT)
+		{
+			tabletSize = priv->bottomX - priv->tvoffsetX;
+			if (v0 > tabletSize && v0 <= priv->bottomX)
+				priv->currentScreen = 0;
+			if (v0 > priv->topX && v0 <= priv->topX + priv->tvoffsetX)
+				priv->currentScreen = 1;
+		}
+		if (priv->twinview == TV_BELOW_ABOVE)
+		{
+			tabletSize = priv->bottomY - priv->tvoffsetY;
+			if (v0 > tabletSize && v0 <= priv->bottomY)
+				priv->currentScreen = 0;
+			if (v0 > priv->topY && v0 <= priv->topY + priv->tvoffsetY)
+				priv->currentScreen = 1;
+		}
+		DBG(10, priv->debugLevel, ErrorF("xf86WcmSetScreen TwinView setup screenToSet=%d\n", 
+			priv->currentScreen));
 	}
 
 	xf86WcmMappingFactor(local);
@@ -219,33 +211,29 @@ static void xf86WcmSetScreen(LocalDevicePtr local, int *value0, int *value1)
 					break;
 			}
 		}
-		if (screenToSet != -1)
-		{
-			letfPadding = priv->screenTopX[screenToSet];
-			topPadding = priv->screenTopY[screenToSet];
-		}
-		else
-		{
-			DBG(3, priv->debugLevel, ErrorF("xf86WcmSetScreen Error: "
-				"Can not find valid screen (currentScreen=%d)\n", 
-				priv->currentScreen));
-			return;
-		}
 	}
 	else
 		screenToSet = priv->screen_no;
 
-	x = (double)v0 * priv->factorX - letfPadding + 0.5;
-	y = (double)v1 * priv->factorY - topPadding + 0.5;
-		
+	if (screenToSet == -1)
+	{
+		DBG(3, priv->debugLevel, ErrorF("xf86WcmSetScreen Error: "
+			"Can not find valid screen (currentScreen=%d)\n", 
+			priv->currentScreen));
+		return;
+	}
+
+	xf86WcmVirtaulTabletPadding(local);
+	x = ((double)(v0 + priv->leftPadding) * priv->factorX) - priv->screenTopX[screenToSet] + 0.5;
+	y = ((double)(v1 + priv->topPadding) * priv->factorY) - priv->screenTopY[screenToSet] + 0.5;
+
 	if (x >= screenInfo.screens[screenToSet]->width)
 		x = screenInfo.screens[screenToSet]->width - 1;
 	if (y >= screenInfo.screens[screenToSet]->height)
 		y = screenInfo.screens[screenToSet]->height - 1;
 
 	xf86XInputSetScreen(local, screenToSet, x, y);
-	DBG(10, priv->debugLevel, ErrorF("xf86WcmSetScreen"
-			" current=%d ToSet=%d\n", 
+	DBG(10, priv->debugLevel, ErrorF("xf86WcmSetScreen current=%d ToSet=%d\n", 
 			priv->currentScreen, screenToSet));
 	priv->currentScreen = screenToSet;
 }
@@ -266,7 +254,7 @@ static void xf86WcmSendButtons(LocalDevicePtr local, int buttons, int rx, int ry
 		"buttons=%d for %s\n", buttons, local->name));
 
 	/* Tablet PC buttons. */
-	if ( common->wcmTPCButton && !IsCursor(priv) && !IsPad(priv) )
+	if ( common->wcmTPCButton && !IsCursor(priv) && !IsPad(priv) && !IsTouch(priv) )
 	{
 		if ( buttons & 1 )
 		{
@@ -515,21 +503,9 @@ static void sendAButton(LocalDevicePtr local, int button, int mask,
 #if WCM_XINPUTABI_MAJOR == 0
 	int is_core = local->flags & (XI86_ALWAYS_CORE | XI86_CORE_POINTER);
 #endif
-	int button_idx, naxes = priv->naxes;
+	int naxes = priv->naxes;
 
-	if (IsPad(priv))
-		button -= 8;
-
-	if (button < 0 || button >= priv->nbuttons)
-	{
-		/* should never happen */
-		ErrorF ("sendAButton: Invalid button index %d (number of defined buttons = %d)\n", button, priv->nbuttons);
-		return; 
-	}
-
-	button_idx = button;
-	button = priv->button[button];
-	if (!button)  /* ignore this button event */
+	if (!priv->button[button])  /* ignore this button event */
 		return;
 
 #if WCM_XINPUTABI_MAJOR == 0
@@ -542,23 +518,18 @@ static void sendAButton(LocalDevicePtr local, int button, int mask,
 		"sendAButton TPCButton(%s) button=%d state=%d " 
 		"code=%08x, for %s coreEvent=%s \n", 
 		common->wcmTPCButton ? "on" : "off", 
-		button_idx, mask, button, 
-		local->name, (button & AC_CORE) ? "yes" : "no"));
+		button, mask, priv->button[button], 
+		local->name, (priv->button[button] & AC_CORE) ? "yes" : "no"));
 
-	switch (button & AC_TYPE)
+	switch (priv->button[button] & AC_TYPE)
 	{
 	case AC_BUTTON:
-/*		xf86PostButtonEvent(local->dev, is_absolute, button & AC_CODE,
-*/		/* Dynamically modify the button map as required --
-		 * to be moved in the place where button mappings are changed
-		 */
-		local->dev->button->map [button_idx] = button & AC_CODE;
-		xf86PostButtonEvent(local->dev, is_absolute, button_idx,
+		xf86PostButtonEvent(local->dev, is_absolute, priv->button[button] & AC_CODE,
 			mask != 0,0,naxes,rx,ry,rz,v3,v4,v5);
 		break;
 
 	case AC_KEY:
-		sendKeystroke(local, button, priv->keys[button_idx], mask);
+		sendKeystroke(local, priv->button[button], priv->keys[button], mask);
 		break;
 
 	case AC_MODETOGGLE:
@@ -574,7 +545,7 @@ static void sendAButton(LocalDevicePtr local, int button, int mask,
 	case AC_DISPLAYTOGGLE:
 		if (mask && priv->numScreen > 1)
 		{
-			if (IsPad(priv)) /* toggle display for all tools except pad */
+			if (IsPad(priv)) /* toggle display (individual screens plus the whole desktop) for all tools except pad */
 			{
 				WacomDevicePtr tmppriv;
 				for (tmppriv = common->wcmDevices; tmppriv; tmppriv = tmppriv->next)
@@ -588,7 +559,7 @@ static void sendAButton(LocalDevicePtr local, int button, int mask,
 					}
 				}
 			}
-			else /* toggle display only for the selected tool */
+			else /* toggle display (individual screens plus the whole desktop) only for the selected tool */
 			{
 				int screen = priv->screen_no;
 				if (++screen >= priv->numScreen)
@@ -598,28 +569,46 @@ static void sendAButton(LocalDevicePtr local, int button, int mask,
 		}
 		break;
 
-	case AC_DBLCLICK:
-		/* Dynamically modify the button map as required --
-		 * to be moved in the place where button mappings are changed.
-		 * Only left double is supported.
-		 */
-		local->dev->button->map [button_idx] = 1;
+	case AC_SCREENTOGGLE:
+		if (mask && priv->numScreen > 1)
+		{
+			if (IsPad(priv)) /* toggle screens for all tools except pad */
+			{
+				WacomDevicePtr tmppriv;
+				for (tmppriv = common->wcmDevices; tmppriv; tmppriv = tmppriv->next)
+				{
+					if (!IsPad(tmppriv))
+					{
+						int screen = tmppriv->screen_no;
+						if (++screen >= tmppriv->numScreen)
+							screen = 0;
+						xf86WcmChangeScreen(tmppriv->local, screen);
+					}
+				}
+			}
+			else /* toggle screens only for the selected tool */
+			{
+				int screen = priv->screen_no;
+				if (++screen >= priv->numScreen)
+					screen = 0;
+				xf86WcmChangeScreen(local, screen);
+			}
+		}
+		break;
 
+	case AC_DBLCLICK:
 		if (mask)
 		{
 			/* Left button down */
 			xf86PostButtonEvent(local->dev, is_absolute,
-				button_idx, 1,0,naxes, 
-				rx,ry,rz,v3,v4,v5);
-
+				1,1,0,naxes, rx,ry,rz,v3,v4,v5);
 			/* Left button up */
 			xf86PostButtonEvent(local->dev, is_absolute,
-				button_idx,0,0,naxes,
-				rx,ry,rz,v3,v4,v5);
+				1,0,0,naxes,rx,ry,rz,v3,v4,v5);
 		}
 
 		/* Left button down/up upon mask is 1/0 */
-		xf86PostButtonEvent(local->dev, is_absolute, button_idx, 
+		xf86PostButtonEvent(local->dev, is_absolute, 1, 
 			mask != 0,0,naxes,rx,ry,rz,v3,v4,v5);
 		break;
 	}
@@ -636,9 +625,11 @@ static void sendAButton(LocalDevicePtr local, int button, int mask,
  *   Send events defined for relative/absolute wheels or strips
  ****************************************************************************/
 
-static void sendWheelStripEvents(LocalDevicePtr local, const WacomDeviceState* ds, int x, int y, int z, int v3, int v4, int v5)
+static void sendWheelStripEvents(LocalDevicePtr local, const WacomDeviceState* ds, 
+		int x, int y, int z, int v3, int v4, int v5)
 {
 	WacomDevicePtr priv = (WacomDevicePtr) local->private;
+	WacomCommonPtr common = priv->common;
 	int fakeButton = 0, i, value = 0, naxes = priv->naxes;
 	unsigned  *keyP = 0;
 	int is_absolute = priv->flags & ABSOLUTE_FLAG;
@@ -667,6 +658,19 @@ static void sendWheelStripEvents(LocalDevicePtr local, const WacomDeviceState* d
 	if ( ds->abswheel != priv->oldWheel )
 	{
 		value = priv->oldWheel - ds->abswheel;
+		if ((strstr(common->wcmModel->name, "Bamboo") ||
+		strstr(common->wcmModel->name, "Intuos4"))
+			&& IsPad(priv))
+		{
+			/* deal with MAX_FINGER_WHEEL to 0 and 0 to MAX_FINGER_WHEEL switching */
+			if (abs(priv->oldWheel - ds->abswheel) > (MAX_FINGER_WHEEL/2))
+			{
+				if (priv->oldWheel > ds->abswheel)
+					value -= MAX_FINGER_WHEEL;
+				else
+					value += MAX_FINGER_WHEEL;
+			}
+		}
 		if ( value > 0 )
 		{
 			fakeButton = priv->wheelup;
@@ -747,13 +751,11 @@ static void sendWheelStripEvents(LocalDevicePtr local, const WacomDeviceState* d
 	switch (fakeButton & AC_TYPE)
 	{
 	    case AC_BUTTON:
-		/* send both button on/off in the same event for pad */
-		local->dev->button->map [0] = fakeButton & AC_CODE;
-	
-		xf86PostButtonEvent(local->dev, is_absolute, 0,
+		/* send both button on/off in the same event for pad */	
+		xf86PostButtonEvent(local->dev, is_absolute, fakeButton & AC_CODE,
 			1,0,naxes,x,y,z,v3,v4,v5);
 
-		xf86PostButtonEvent(local->dev, is_absolute, 0,
+		xf86PostButtonEvent(local->dev, is_absolute, fakeButton & AC_CODE,
 			0,0,naxes,x,y,z,v3,v4,v5);
 	    break;
 
@@ -784,6 +786,7 @@ static void sendCommonEvents(LocalDevicePtr local, const WacomDeviceState* ds, i
 	WacomDevicePtr priv = (WacomDevicePtr) local->private;
 	int buttons = ds->buttons;
 
+	/* send button events when state changed or first time in prox and button unpresses */
 	if (priv->oldButtons != buttons || (!priv->oldProximity && !buttons))
 		xf86WcmSendButtons(local,buttons,x,y,z,v3,v4,v5);
 
@@ -827,7 +830,7 @@ void xf86WcmSendEvents(LocalDevicePtr local, const WacomDeviceState* ds)
 	if (priv->serial && serial != priv->serial)
 	{
 		DBG(10, priv->debugLevel, ErrorF("[%s] serial number"
-			" is %d but your system configured %d", 
+			" is %u but your system configured %u", 
 			local->name, serial, (int)priv->serial));
 		return;
 	}
@@ -873,7 +876,7 @@ void xf86WcmSendEvents(LocalDevicePtr local, const WacomDeviceState* ds)
 		v3 = rot;
 		v4 = throttle;
 	}
-	else
+	else  /* Intuos styli have tilt */
 	{
 		v3 = tx;
 		v4 = ty;
@@ -882,7 +885,7 @@ void xf86WcmSendEvents(LocalDevicePtr local, const WacomDeviceState* ds)
 
 	DBG(6, priv->debugLevel, ErrorF("[%s] %s prox=%d\tx=%d"
 		"\ty=%d\tz=%d\tv3=%d\tv4=%d\tv5=%d\tid=%d"
-		"\tserial=%d\tbutton=%s\tbuttons=%d\n",
+		"\tserial=%u\tbutton=%s\tbuttons=%d\n",
 		local->name,
 		is_absolute ? "abs" : "rel",
 		is_proximity,
@@ -951,7 +954,7 @@ void xf86WcmSendEvents(LocalDevicePtr local, const WacomDeviceState* ds)
 			 * screen and modify the axes before posting events */
 			if(!(priv->flags & BUTTONS_ONLY_FLAG))
 			{
-				xf86WcmSetScreen(local, &x, &y);
+				xf86WcmSetScreen(local, x, y);
 			}
 
 			/* unify acceleration in both directions 
@@ -959,6 +962,13 @@ void xf86WcmSendEvents(LocalDevicePtr local, const WacomDeviceState* ds)
 			 */
 			if (!is_absolute)
 				x *= priv->factorY / priv->factorX;
+ 			else
+			{
+				/* Padding virtual values */
+				xf86WcmVirtaulTabletPadding(local);
+				x += priv->leftPadding;
+				y += priv->topPadding;
+			}
 
 #ifdef WCM_XORG_TABLET_SCALING
 			/* Ugly hack for Xorg 7.3, which doesn't call xf86WcmDevConvert
@@ -969,15 +979,30 @@ void xf86WcmSendEvents(LocalDevicePtr local, const WacomDeviceState* ds)
  			if(is_absolute) {
 				x -= priv->topX;
 				y -= priv->topY;
+				if (priv->currentScreen == 1 && priv->twinview > TV_XINERAMA)
+				{
+					x -= priv->tvoffsetX;
+					y -= priv->tvoffsetY;
+				}
 			}
 			x = (int)((double)x * priv->factorX + (x>=0?0.4:-0.4));
 			y = (int)((double)y * priv->factorY + (y>=0?0.4:-0.4));
 
-			/* map to a specific screen */
-			if (priv->screen_no != -1 || priv->twinview != TV_NONE)
+			if ((priv->flags & ABSOLUTE_FLAG) && (priv->twinview <= TV_XINERAMA))
 			{
-				x += priv->screenTopX[priv->currentScreen];
-				y += priv->screenTopY[priv->currentScreen];
+				x -= priv->screenTopX[priv->currentScreen];
+				y -= priv->screenTopY[priv->currentScreen];
+			}
+
+			if (priv->screen_no != -1)
+			{
+				if (x > priv->screenBottomX[priv->currentScreen] - priv->screenTopX[priv->currentScreen])
+					x = priv->screenBottomX[priv->currentScreen];
+				if (x < 0) x = 0;
+				if (y > priv->screenBottomY[priv->currentScreen] - priv->screenTopY[priv->currentScreen])
+					y = priv->screenBottomY[priv->currentScreen];
+				if (y < 0) y = 0;
+	
 			}
 			priv->currentSX = x;
 			priv->currentSY = y;
@@ -1017,12 +1042,13 @@ void xf86WcmSendEvents(LocalDevicePtr local, const WacomDeviceState* ds)
 	}
 	else
 	{
+
 		if (v3 || v4 || v5 || buttons || ds->relwheel)
 		{
 			x = 0;
 			y = 0;
 			if ( v3 || v4 || v5 )
-				xf86WcmSetScreen(local, &x, &y);
+				xf86WcmSetScreen(local, x, y);
 
 			/* don't emit proximity events if device does not support proximity */
 			if ((local->dev->proximity && !priv->oldProximity))
@@ -1117,11 +1143,14 @@ static int xf86WcmSuppress(WacomCommonPtr common, const WacomDeviceState* dsOrig
 	if ((ABS(dsOrig->abswheel - dsNew->abswheel) > suppress) 
 		|| (dsNew->relwheel != 0)) returnV = 1;
 
-	/* need to check if cursor moves or not */
+	/* cursor moves or not? */
 	if ((ABS(dsOrig->x - dsNew->x) > suppress) || 
 			(ABS(dsOrig->y - dsNew->y) > suppress)) 
-		returnV = 2;
-	else if (returnV == 1) /* don't move the cursor */
+	{
+		if (!returnV) /* need to check if cursor moves or not */
+			returnV = 2;
+	}
+	else /* don't move cursor */
 	{
 		dsNew->x = dsOrig->x;
 		dsNew->y = dsOrig->y;
@@ -1234,7 +1263,7 @@ void xf86WcmEvent(WacomCommonPtr common, unsigned int channel,
 #ifdef WCM_ENABLE_LINUXINPUT
 	/* Discard the first 2 USB packages due to events delay */
 	if ( (pChannel->nSamples < 2) && (common->wcmDevCls == &gWacomUSBDevice) && 
-		ds.device_type != PAD_ID )
+		(ds.device_type != PAD_ID) )
 	{
 		DBG(11, common->debugLevel, 
 			ErrorF("discarded %dth USB data.\n", 
@@ -1243,6 +1272,13 @@ void xf86WcmEvent(WacomCommonPtr common, unsigned int channel,
 		return; /* discard */
 	}
 #endif
+
+	if (strstr(common->wcmModel->name, "Intuos4"))
+	{
+		/* convert Intuos4 mouse tilt to rotation */
+		xf86WcmTilt2R(&ds);
+	}
+
 	fs = &pChannel->rawFilter;
 	if (!fs->npoints && ds.proximity)
 	{
@@ -1265,7 +1301,7 @@ void xf86WcmEvent(WacomCommonPtr common, unsigned int channel,
 		}
 		fs->x[0] = ds.x;
 		fs->y[0] = ds.y;
-		if (HANDLE_TILT(common) && (ds.device_type == STYLUS_ID || ds.device_type == ERASER_ID))
+		if (HANDLE_TILT(common) && ((ds.device_type == STYLUS_ID) || (ds.device_type == ERASER_ID)))
 		{
 			for (i=common->wcmRawSample - 1; i>0; i--)
 			{
@@ -1275,7 +1311,8 @@ void xf86WcmEvent(WacomCommonPtr common, unsigned int channel,
 			fs->tiltx[0] = ds.tiltx;
 			fs->tilty[0] = ds.tilty;
 		}
-		if (RAW_FILTERING(common) && common->wcmModel->FilterRaw && ds.device_type != PAD_ID)
+
+		if (RAW_FILTERING(common) && common->wcmModel->FilterRaw && (ds.device_type != PAD_ID))
 		{
 			if (common->wcmModel->FilterRaw(common,pChannel,&ds))
 			{
@@ -1285,7 +1322,6 @@ void xf86WcmEvent(WacomCommonPtr common, unsigned int channel,
 				return; /* discard */
 			}
 		}
-
 		/* Discard unwanted data */
 		suppress = xf86WcmSuppress(common, pLast, &ds);
 		if (!suppress)
@@ -1306,8 +1342,8 @@ void xf86WcmEvent(WacomCommonPtr common, unsigned int channel,
 		pChannel->valid.states,
 		sizeof(WacomDeviceState) * (common->wcmRawSample - 1));
 	pChannel->valid.state = ds; /*save last raw sample */
-	if (pChannel->nSamples < common->wcmRawSample) ++pChannel->nSamples;
 
+	if (pChannel->nSamples < common->wcmRawSample) ++pChannel->nSamples;
 	commonDispatchDevice(common,channel,pChannel, suppress);
 	resetSampleCounter(pChannel);
 }
@@ -1390,7 +1426,6 @@ static void commonDispatchDevice(WacomCommonPtr common, unsigned int channel,
 					break;
 				}
 	}
-
 	DBG(10, common->debugLevel, ErrorF("commonDispatchDevice device type = %d\n", ds->device_type));
 	/* Find the device the current events are meant for */
 	/* 1: Find the tool (the one with correct serial or in second
@@ -1480,10 +1515,8 @@ static void commonDispatchDevice(WacomCommonPtr common, unsigned int channel,
 	if (pDev && !miPointerGetScreen(pDev->dev))
 #endif
 	{
-		DBG(1, common->debugLevel, ErrorF("xf86WcmEvent: "
-			"Wacom driver can not get Current Screen ID\n"));
-		DBG(1, common->debugLevel, ErrorF(
-			"Please remove Wacom tool from the tablet.\n"));
+		ErrorF("xf86WcmEvent: Wacom driver can not get Current Screen ID\n");
+		ErrorF("Please remove Wacom tool from the tablet and bring it back again.\n");
 		return;
 	}
 
@@ -1543,10 +1576,18 @@ static void commonDispatchDevice(WacomCommonPtr common, unsigned int channel,
 
 		if (IsStylus(priv) || IsEraser(priv))
 		{
-			if (filtered.pressure < common->wcmThreshold )
-				filtered.buttons &= ~button;
-			else
+			/* set button1 (left click) on/off */
+			if (filtered.pressure >= common->wcmThreshold)
 				filtered.buttons |= button;
+			else
+			{
+				/* threshold tolerance */
+				int tol = common->wcmMaxZ / 250;
+				if (strstr(common->wcmModel->name, "Intuos4"))
+					tol = common->wcmMaxZ / 125;
+				if (filtered.pressure < common->wcmThreshold - tol)
+					filtered.buttons &= ~button;
+			}
 			/* transform pressure */
 			transPressureCurve(priv,&filtered);
 		}
@@ -1559,7 +1600,7 @@ static void commonDispatchDevice(WacomCommonPtr common, unsigned int channel,
 					(5 - common->wcmCapacity))
 				filtered.buttons |= button;
 		}
-		else if (!(priv->flags & ABSOLUTE_FLAG) && !priv->hardProx)
+		else if (IsCursor(priv) && !priv->hardProx)
 		{
 			/* initial current max distance */
 			if (strstr(common->wcmModel->name, "Intuos"))
@@ -1620,21 +1661,31 @@ static void commonDispatchDevice(WacomCommonPtr common, unsigned int channel,
 
 		#endif /* throttle */
 
-		if (!(priv->flags & ABSOLUTE_FLAG) && (suppress == 2))
+		if (!(priv->flags & ABSOLUTE_FLAG) && !IsPad(priv))
 		{
 			/* To improve the accuracy of relative x/y,
-			 * don't send event when there is no movement.
+			 * don't send motion event when there is no movement.
 			 */
 			double deltx = filtered.x - priv->oldX;
 			double delty = filtered.y - priv->oldY;
 			deltx *= priv->factorY*priv->speed;
 			delty *= priv->factorY*priv->speed;
-
+	
 			if (ABS(deltx)<1 && ABS(delty)<1) 
 			{
-				DBG(10, common->debugLevel, ErrorF(
-					"Ignore non-movement relative data \n"));
-				return;
+				/* don't move the cursor */
+				if (suppress == 1) 
+				{
+					/* send other events, such as button/wheel */
+					filtered.x = priv->oldX;
+					filtered.y = priv->oldY;
+				}
+				else /* no other events to send */
+				{
+					DBG(10, common->debugLevel, ErrorF(
+						"Ignore non-movement relative data \n"));
+					return;
+				}
 			}
 			else
 			{
@@ -1647,10 +1698,11 @@ static void commonDispatchDevice(WacomCommonPtr common, unsigned int channel,
 			}
 		}
 
-		/* force out-prox when distance is outside wcmCursorProxoutDist. */
-		if (!(priv->flags & ABSOLUTE_FLAG))
+		/* force out-prox when distance is outside wcmCursorProxoutDist for pucks */
+		if (IsCursor(priv))
 		{
-			if (strstr(common->wcmModel->name, "Intuos"))
+			/* force out-prox when distance is outside wcmCursorProxoutDist. */
+			if (common->wcmProtocolLevel == 5)
 			{
 				if (common->wcmMaxCursorDist > filtered.distance)
 					common->wcmMaxCursorDist = filtered.distance;
@@ -1695,7 +1747,7 @@ static void commonDispatchDevice(WacomCommonPtr common, unsigned int channel,
 	else
 	{
 		DBG(11, common->debugLevel, ErrorF("no device matches with"
-				" id=%d, serial=%d\n",
+				" id=%d, serial=%u\n",
 				ds->device_type, ds->serial_num));
 	}
 }
@@ -1725,7 +1777,10 @@ int xf86WcmInitTablet(LocalDevicePtr local, const char* id, float version)
 	if (common->wcmThreshold <= 0)
 	{
 		/* Threshold for counting pressure as a button */
-		common->wcmThreshold = common->wcmMaxZ * 3 / 50;
+		if (strstr(common->wcmModel->name, "Intuos4"))
+			common->wcmThreshold = common->wcmMaxZ * 3 / 25;
+		else
+			common->wcmThreshold = common->wcmMaxZ * 3 / 50;
 		ErrorF("%s Wacom using pressure threshold of %d for button 1\n",
 			XCONFIG_PROBED, common->wcmThreshold);
 	}
@@ -1817,12 +1872,12 @@ static void xf86WcmInitialTVScreens(LocalDevicePtr local)
 {
 	WacomDevicePtr priv = (WacomDevicePtr)local->private;
 
-	if (priv->twinview == TV_NONE)
+	if (priv->twinview <= TV_XINERAMA)
 		return;
 
 	priv->numScreen = 2;
 
-	if (priv->twinview == TV_LEFT_RIGHT)
+	if ((priv->twinview == TV_LEFT_RIGHT) || (priv->twinview == TV_RIGHT_LEFT))
 	{
 		/* it does not need the offset if always map to a specific screen */
 		if (priv->screen_no == -1)
@@ -1840,7 +1895,7 @@ static void xf86WcmInitialTVScreens(LocalDevicePtr local)
 			priv->tvResolution[3] = priv->tvResolution[1];
 		}
 	}
-	else if (priv->twinview == TV_ABOVE_BELOW)
+	else if ((priv->twinview == TV_ABOVE_BELOW) || (priv->twinview == TV_BELOW_ABOVE))
 	{
 		/* it does not need the offset if always map to a specific screen */
 		if (priv->screen_no == -1)
@@ -1860,12 +1915,12 @@ static void xf86WcmInitialTVScreens(LocalDevicePtr local)
 	}
 
 	/* initial screen info */
-	priv->screenTopX[0] = 0;
-	priv->screenTopY[0] = 0;
-	priv->screenBottomX[0] = priv->tvResolution[0];
-	priv->screenBottomY[0] = priv->tvResolution[1];
 	if (priv->twinview == TV_ABOVE_BELOW)
 	{
+		priv->screenTopX[0] = 0;
+		priv->screenTopY[0] = 0;
+		priv->screenBottomX[0] = priv->tvResolution[0];
+		priv->screenBottomY[0] = priv->tvResolution[1];
 		priv->screenTopX[1] = 0;
 		priv->screenTopY[1] = priv->tvResolution[1];
 		priv->screenBottomX[1] = priv->tvResolution[2];
@@ -1873,11 +1928,37 @@ static void xf86WcmInitialTVScreens(LocalDevicePtr local)
 	}
 	if (priv->twinview == TV_LEFT_RIGHT)
 	{
+		priv->screenTopX[0] = 0;
+		priv->screenTopY[0] = 0;
+		priv->screenBottomX[0] = priv->tvResolution[0];
+		priv->screenBottomY[0] = priv->tvResolution[1];
 		priv->screenTopX[1] = priv->tvResolution[0];
 		priv->screenTopY[1] = 0;
 		priv->screenBottomX[1] = priv->tvResolution[0] + priv->tvResolution[2];
 		priv->screenBottomY[1] = priv->tvResolution[3];
 	}
+	if (priv->twinview == TV_BELOW_ABOVE)
+	{
+		priv->screenTopX[0] = 0;
+		priv->screenTopY[0] = priv->tvResolution[1];
+		priv->screenBottomX[0] = priv->tvResolution[2];
+		priv->screenBottomY[0] = priv->tvResolution[1] + priv->tvResolution[3];
+		priv->screenTopX[1] = 0;
+		priv->screenTopY[1] = 0;
+		priv->screenBottomX[1] = priv->tvResolution[0];
+		priv->screenBottomY[1] = priv->tvResolution[1];
+	}
+	if (priv->twinview == TV_RIGHT_LEFT)
+	{
+		priv->screenTopX[0] = priv->tvResolution[0];
+		priv->screenTopY[0] = 0;
+		priv->screenBottomX[0] = priv->tvResolution[0] + priv->tvResolution[2];
+		priv->screenBottomY[0] = priv->tvResolution[3];
+		priv->screenTopX[1] = 0;
+		priv->screenTopY[1] = 0;
+		priv->screenBottomX[1] = priv->tvResolution[0];
+		priv->screenBottomY[1] = priv->tvResolution[1];
+	}
 
 	DBG(10, priv->debugLevel, ErrorF("xf86WcmInitialTVScreens for \"%s\" "
 		"topX0=%d topY0=%d bottomX0=%d bottomY0=%d "
@@ -1901,7 +1982,7 @@ void xf86WcmInitialScreens(LocalDevicePtr local)
 		"number of screen=%d \n", local->name, screenInfo.numScreens));
 	priv->tvoffsetX = 0;
 	priv->tvoffsetY = 0;
-	if (priv->twinview != TV_NONE)
+	if (priv->twinview > TV_XINERAMA)
 	{
 		xf86WcmInitialTVScreens(local);
 		return;
@@ -1993,7 +2074,6 @@ static void rotateOneTool(WacomDevicePtr priv)
 		area->bottomY = priv->bottomY = oldMaxY - tmpTopY;
 		break;
 	}
-	xf86WcmInitialScreens(priv->local);
 	xf86WcmMappingFactor(priv->local);
 	xf86WcmInitialCoordinates(priv->local, 0);
 	xf86WcmInitialCoordinates(priv->local, 1);
@@ -2009,10 +2089,10 @@ static void rotateOneTool(WacomDevicePtr priv)
 }
 
 /*****************************************************************************
- * xf86WcmRotateScreen
+ * xf86WcmRotateTablet
  ****************************************************************************/
 
-void xf86WcmRotateScreen(LocalDevicePtr local, int value)
+void xf86WcmRotateTablet(LocalDevicePtr local, int value)
 {
 	WacomDevicePtr priv = (WacomDevicePtr)local->private;
 	WacomCommonPtr common = priv->common;
@@ -2020,7 +2100,7 @@ void xf86WcmRotateScreen(LocalDevicePtr local, int value)
 	int oldRotation;
 	int tmpTopX, tmpTopY, tmpBottomX, tmpBottomY, oldMaxX, oldMaxY;
 
-	DBG(10, priv->debugLevel, ErrorF("xf86WcmRotateScreen for \"%s\" \n", local->name));
+	DBG(10, priv->debugLevel, ErrorF("xf86WcmRotateTablet for \"%s\" \n", local->name));
 
 	if (common->wcmRotate == value) /* initialization */
 	{
@@ -2089,5 +2169,4 @@ void xf86WcmRotateScreen(LocalDevicePtr local, int value)
 		    }
 		}
 	}
-
 }
