@@ -1,6 +1,6 @@
 /*
  * Copyright 1995-2002 by Frederic Lepied, France. <Lepied@XFree86.org>
- * Copyright 2002-2008 by Ping Cheng, Wacom Technology. <pingc@wacom.com>		
+ * Copyright 2002-2009 by Ping Cheng, Wacom Technology. <pingc@wacom.com>		
  *                                                                            
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -45,6 +45,10 @@
 #define BTN_TASK 0x117
 #endif
 
+#ifndef BTN_TOOL_DOUBLETAP
+#define BTN_TOOL_DOUBLETAP 0x14d
+#endif
+
 static Bool usbDetect(LocalDevicePtr);
 Bool usbWcmInit(LocalDevicePtr pDev, char* id, float *version);
 
@@ -212,7 +216,7 @@ static void usbParseChannel(LocalDevicePtr local, int channel, int serial);
 
 	static WacomModel usbCintiq =
 	{
-		"USB Cintiq",
+		"USB PL/Cintiq",
 		usbInitProtocol4,
 		NULL,                 /* resolution not queried */
 		usbWcmGetRanges,
@@ -242,9 +246,9 @@ static void usbParseChannel(LocalDevicePtr local, int channel, int serial);
 		usbDetectConfig,      /* detect hardware buttons etc */
 	};
 
-	static WacomModel usbIntuos =
+	static WacomModel usbIntuos1 =
 	{
-		"USB Intuos",
+		"USB Intuos1",
 		usbInitProtocol5,
 		NULL,                 /* resolution not queried */
 		usbWcmGetRanges,
@@ -290,6 +294,22 @@ static void usbParseChannel(LocalDevicePtr local, int channel, int serial);
 		usbDetectConfig,      /* detect hardware buttons etc */
 	};
 
+	static WacomModel usbIntuos4 =
+	{
+		"USB Intuos4",
+		usbInitProtocol5,
+		NULL,                 /* resolution not queried */
+		usbWcmGetRanges,
+		NULL,                 /* reset not supported */
+		NULL,                 /* tilt automatically enabled */
+		NULL,                 /* suppress implemented in software */
+		NULL,                 /* link speed unsupported */
+		NULL,                 /* start not supported */
+		usbParse,
+		xf86WcmFilterIntuos,  /* input filtering recommended */
+		usbDetectConfig,      /* detect hardware buttons etc */
+	};
+
 	static WacomModel usbVolito =
 	{
 		"USB Volito",
@@ -430,11 +450,11 @@ static struct
 	{ 0x19, 2032, 2032, &usbBamboo1    }, /* Bamboo1 Medium*/ 
 	{ 0x81, 2032, 2032, &usbGraphire4  }, /* Graphire4 6x8 BlueTooth */
 
-	{ 0x20, 2540, 2540, &usbIntuos     }, /* Intuos 4x5 */
-	{ 0x21, 2540, 2540, &usbIntuos     }, /* Intuos 6x8 */
-	{ 0x22, 2540, 2540, &usbIntuos     }, /* Intuos 9x12 */
-	{ 0x23, 2540, 2540, &usbIntuos     }, /* Intuos 12x12 */
-	{ 0x24, 2540, 2540, &usbIntuos     }, /* Intuos 12x18 */
+	{ 0x20, 2540, 2540, &usbIntuos1    }, /* Intuos 4x5 */
+	{ 0x21, 2540, 2540, &usbIntuos1    }, /* Intuos 6x8 */
+	{ 0x22, 2540, 2540, &usbIntuos1    }, /* Intuos 9x12 */
+	{ 0x23, 2540, 2540, &usbIntuos1    }, /* Intuos 12x12 */
+	{ 0x24, 2540, 2540, &usbIntuos1    }, /* Intuos 12x18 */
 
 	{ 0x03,  508,  508, &usbCintiqPartner }, /* PTU600 */
 
@@ -448,6 +468,7 @@ static struct
 	{ 0x38,  508,  508, &usbCintiq     }, /* PL510 */
 	{ 0x39,  508,  508, &usbCintiq     }, /* PL710 */ 
 	{ 0xC0,  508,  508, &usbCintiq     }, /* DTF720 */
+	{ 0xC2,  508,  508, &usbCintiq     }, /* DTF720a */
 	{ 0xC4,  508,  508, &usbCintiq     }, /* DTF521 */ 
 	{ 0xC7, 2540, 2540, &usbCintiq     }, /* DTU1931 */
 
@@ -476,6 +497,11 @@ static struct
 	{ 0xB5, 5080, 5080, &usbIntuos3    }, /* Intuos3 6x11 */
 	{ 0xB7, 5080, 5080, &usbIntuos3    }, /* Intuos3 4x6 */
 
+	{ 0xB8, 5080, 5080, &usbIntuos4    }, /* Intuos4 4x6 */
+	{ 0xB9, 5080, 5080, &usbIntuos4    }, /* Intuos4 6x9 */
+	{ 0xBA, 5080, 5080, &usbIntuos4    }, /* Intuos4 8x13 */
+	{ 0xBB, 5080, 5080, &usbIntuos4    }, /* Intuos4 12x19*/
+
 	{ 0x3F, 5080, 5080, &usbCintiqV5   }, /* Cintiq 21UX */ 
 	{ 0xC5, 5080, 5080, &usbCintiqV5   }, /* Cintiq 20WSX */ 
 	{ 0xC6, 5080, 5080, &usbCintiqV5   }, /* Cintiq 12WX */ 
@@ -524,25 +550,38 @@ Bool usbWcmInit(LocalDevicePtr local, char* id, float *version)
 			common->wcmCapacityDefault = -1; 
 		}
 
-		if (common->tablet_id == 0x9A || common->tablet_id == 0x93)
+		if (common->tablet_id == 0x9A || common->tablet_id == 0x93 || common->tablet_id == 0x90)
 		{
-			/* TouchDefault was off for all devices */
-			/* except when touch is supported */
-			common->wcmTouchDefault = 1;
-			common->wcmTPCButtonDefault = 1; /* Tablet PC buttons on by default */
+			if (common->tablet_id != 0x90)
+			{
+				/* TouchDefault was off for all devices */
+				/* except when touch is supported */
+				common->wcmTouchDefault = 1;
 
-			/* check if touch was turned off in xorg.conf */
-			common->wcmTouch = xf86SetBoolOption(local->options, "Touch", common->wcmTouchDefault);
-			if ( common->wcmTouch )
-				xf86Msg(X_CONFIG, "%s: Touch is enabled \n", common->wcmDevice);
+				/* check if touch was turned off in xorg.conf */
+				common->wcmTouch = xf86SetBoolOption(local->options, 
+						"Touch", common->wcmTouchDefault);
+				if ( common->wcmTouch )
+					xf86Msg(X_CONFIG, "%s: Touch is enabled \n", common->wcmDevice);
+			}
 
 			/* Tablet PC button applied to the whole tablet. Not just one tool */
-			common->wcmTPCButton = xf86SetBoolOption(local->options, 
+			common->wcmTPCButtonDefault = 1; /* Tablet PC buttons on by default */
+			if ( priv->flags & STYLUS_ID )
+			{
+				common->wcmTPCButton = xf86SetBoolOption(local->options, 
 					"TPCButton", common->wcmTPCButtonDefault);
-			if ( common->wcmTPCButton )
-				xf86Msg(X_CONFIG, "%s: Tablet PC buttons are on \n", common->wcmDevice);
+				if ( common->wcmTPCButton )
+					xf86Msg(X_CONFIG, "%s: Tablet PC buttons are on \n", 
+						common->wcmDevice);
+			}
 		}
 	}
+	else
+	{
+		ErrorF("%x is not supported by linuxwacom.\n", sID[1]);
+		return FALSE;
+	}
 
 	if (!common->wcmModel)
 	{
@@ -750,7 +789,7 @@ static int usbDetectConfig(LocalDevicePtr local)
 		priv->nbuttons = common->npadkeys;
 
 /* This code will be used when we are ready to report valuators in tablet and tool 
- * specific form, whcih will need to clean InitValuatorAxisStruct() in xf86Wacom.c
+ * specific form, which will need to clean InitValuatorAxisStruct() in xf86Wacom.c
  * and all the calls to X that are related to valuators, such as xf86PostButtonEvent and 
  * xf86PostButtonEvent, etc. Code under util directory will need to be updated as well.
  * This will take some time. We put it in the to-do list for now.  Ping 
@@ -802,8 +841,7 @@ static void usbParseEvent(LocalDevicePtr local,
 	{
 		ErrorF("usbParse: Exceeded event queue (%d) \n",
 				common->wcmEventCnt);
-		common->wcmEventCnt = 0;
-		common->wcmLastToolSerial = 0;
+		goto skipEvent;
 		return;
 	}
 
@@ -818,6 +856,7 @@ static void usbParseEvent(LocalDevicePtr local,
 		if (event->value == 0) /* serial number should never be 0 */
 		{
 			ErrorF("usbParse: Ignoring event from invalid serial 0\n");
+			goto skipEvent;
 			return;
 		}
 		common->wcmLastToolSerial = event->value;
@@ -857,56 +896,88 @@ static void usbParseEvent(LocalDevicePtr local,
 			channel = 1;
 		else
 			channel = 0;
-		if (!common->wcmChannel[channel].work.proximity)
-		{
-			memset(&common->wcmChannel[channel],0,sizeof(WacomChannel));
-			/* in case the in-prox event was missing */
-			common->wcmChannel[channel].work.proximity = 1;
-		}
 	}
 	else if (common->wcmLastToolSerial) /* serial number should never be 0 */
 	{
-		/* find existing channel */
-		for (i=0; i<MAX_CHANNELS; ++i)
+		/* ignore events without information */
+		if (common->wcmEventCnt <= 2) 
 		{
-			if (common->wcmChannel[i].work.proximity &&
-			   common->wcmChannel[i].work.serial_num == common->wcmLastToolSerial)
-			{
-				channel = i;
-				break;
-			}
+			ErrorF("%s - usbParse: dropping empty event for serial %d\n", 
+				local->name, common->wcmLastToolSerial);
+			goto skipEvent;
 		}
 
-		/* find an empty channel */
-		if (channel < 0)
+		/* dual input is supported */
+		if ( strstr(common->wcmModel->name, "Intuos1") || strstr(common->wcmModel->name, "Intuos2") )
 		{
+			/* find existing channel */
 			for (i=0; i<MAX_CHANNELS; ++i)
 			{
-				if (!common->wcmChannel[i].work.proximity)
+				if (common->wcmChannel[i].work.proximity && 
+			  		common->wcmChannel[i].work.serial_num == common->wcmLastToolSerial)
 				{
-					memset(&common->wcmChannel[i],0,
-							sizeof(WacomChannel));
-					/* in case the in-prox event was missing */
-					common->wcmChannel[i].work.proximity = 1;
 					channel = i;
 					break;
 				}
 			}
+
+			/* find an empty channel */
+			if (channel < 0)
+			{
+				for (i=0; i<MAX_CHANNELS; ++i)
+				{
+					if (!common->wcmChannel[i].work.proximity)
+					{
+						channel = i;
+						break;
+					}
+				}
+			}
+		}
+		else  /* one transducer plus expresskey (pad) is supported */
+		{
+			if (common->wcmLastToolSerial == -1)  /* pad */
+				channel = 1;
+			else if ( (common->wcmChannel[0].work.proximity &&  /* existing transducer */
+				    (common->wcmChannel[0].work.serial_num == common->wcmLastToolSerial)) ||
+					!common->wcmChannel[0].work.proximity ) /* new transducer */
+				channel = 0;
+			
 		}
 	}
 
 	/* fresh out of channels */
 	if (channel < 0)
 	{
-		/* this should never happen in normal use */
-		ErrorF("usbParse: Exceeded channel count; "
-			"ignoring the events.\n");
-		common->wcmEventCnt = 0;
-		return;
-
+		/* This should never happen in normal use.
+		 * Let's start over again. Force prox-out for all channels.
+		 */
+		for (i=0; i<MAX_CHANNELS; ++i)
+		{
+			if (common->wcmChannel[i].work.proximity && (common->wcmChannel[i].work.serial_num != -1))
+			{
+				common->wcmChannel[i].work.proximity = 0;
+				/* dispatch event */
+				xf86WcmEvent(common, i, &common->wcmChannel[i].work);
+			}
+		}
+		DBG(1, common->debugLevel, ErrorF("usbParse (%s with serial number: %u) at %d: Exceeded channel count; "
+			"ignoring the events.\n", local->name, common->wcmLastToolSerial, 
+			(int)GetTimeInMillis()));
+		goto skipEvent;
 	}
 
+	if (!common->wcmChannel[channel].work.proximity)
+	{
+		memset(&common->wcmChannel[channel],0,sizeof(WacomChannel));
+		/* in case the in-prox event was missing */
+		common->wcmChannel[channel].work.proximity = 1;
+	}
+
+	/* dispatch event */
 	usbParseChannel(local,channel,common->wcmLastToolSerial);
+
+skipEvent:
 	common->wcmLastToolSerial = 0;
 	common->wcmEventCnt = 0;
 }
@@ -919,6 +990,7 @@ static void usbParseChannel(LocalDevicePtr local, int channel, int serial)
 	WacomDevicePtr priv = (WacomDevicePtr)local->private;
 	WacomCommonPtr common = priv->common;
 
+	DBG(6, common->debugLevel, ErrorF("usbParseChannel %d events received\n", common->wcmEventCnt));
 	#define MOD_BUTTONS(bit, value) do { \
 		shift = 1<<bit; \
 		ds->buttons = (((value) != 0) ? \
@@ -929,7 +1001,6 @@ static void usbParseChannel(LocalDevicePtr local, int channel, int serial)
 		DBG(6, common->debugLevel, ErrorF("usbParseChannel no real events received\n"));
 		return;
 	}
-	DBG(6, common->debugLevel, ErrorF("usbParseChannel %d events received\n", common->wcmEventCnt));
 
 	/* all USB data operates from previous context except relative values*/
 	ds = &common->wcmChannel[channel].work;
@@ -1064,7 +1135,7 @@ static void usbParseChannel(LocalDevicePtr local, int channel, int serial)
 				for (nkeys = 0; nkeys < common->npadkeys; nkeys++)
 					if (event->code == common->padkey_code [nkeys])
 					{
-						MOD_BUTTONS ((MAX_MOUSE_BUTTONS/2+nkeys), event->value);
+						MOD_BUTTONS (nkeys, event->value);
 						break;
 					}
 			}
@@ -1072,15 +1143,15 @@ static void usbParseChannel(LocalDevicePtr local, int channel, int serial)
 	} /* next event */
 
 	/* don't send touch event when touch isn't enabled */
-	if (ds->device_type == TOUCH_ID && !common->wcmTouch)
+	if ((ds->device_type == TOUCH_ID) && !common->wcmTouch)
 		return;
 
 	/* it is an out-prox when id or/and serial number is zero for kernel 2.4 */
 	if ((!ds->device_id || !ds->serial_num) && !USE_SYN_REPORTS(common))
  		ds->proximity = 0;
 
-	/* DTF720 doesn't support eraser */
-	if (common->tablet_id == 0xC0 && ds->device_type == ERASER_ID) 
+	/* DTF720 and DTF720a don't support eraser */
+	if (((common->tablet_id == 0xC0) || (common->tablet_id == 0xC2)) && (ds->device_type == ERASER_ID)) 
 	{
 		DBG(10, common->debugLevel, ErrorF("usbParseChannel "
 			"DTF 720 doesn't support eraser "));
