@@ -1,5 +1,5 @@
 /*
- * Copyright 2007-2008 by Ping Cheng, Wacom Technology. <pingc@wacom.com>
+ * Copyright 2007-2009 by Ping Cheng, Wacom Technology. <pingc@wacom.com>
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -20,8 +20,10 @@
  * REVISION HISTORY
  *
  * 2007-05-25 0.1 - Initial release - span off from xf86Wacom.c
- * 2008-05-14 0.2 - Rotate through routine xf86WcmRotateScreen
+ * 2008-05-14 0.2 - Rotate through routine xf86WcmRotateTablet
  * 2008-06-26 0.3 - Added Capacity
+ * 2009-03-16 0.4 - Added leftOF for TwinView
+ * 2009-04-21 0.5 - Added set serial option
  */
 
 
@@ -31,8 +33,8 @@
 #include "wcmFilter.h"
 
 extern void xf86WcmInitialCoordinates(LocalDevicePtr local, int axes);
+extern void xf86WcmRotateTablet(LocalDevicePtr local, int value);
 extern void xf86WcmInitialScreens(LocalDevicePtr local);
-extern void xf86WcmRotateScreen(LocalDevicePtr local, int value);
 
 /*****************************************************************************
  * xf86WcmSetPadCoreMode
@@ -135,7 +137,6 @@ void xf86WcmChangeScreen(LocalDevicePtr local, int value)
 	if (priv->screen_no != -1)
 		priv->currentScreen = priv->screen_no;
 	xf86WcmInitialScreens(local);
-	xf86WcmMappingFactor(local);
 	xf86WcmInitialCoordinates(local, 0);
 	xf86WcmInitialCoordinates(local, 1);
 }
@@ -326,6 +327,20 @@ static int xf86WcmSetParam(LocalDevicePtr local, int param, int value)
 		xf86ReplaceIntOption(local->options, "Threshold", 
 				common->wcmThreshold);
 		break;
+	    case XWACOM_PARAM_THRESHOLD:
+		common->wcmThreshold = value;
+		xf86ReplaceIntOption(local->options, "Threshold", 
+				common->wcmThreshold);
+		break;
+	    case XWACOM_PARAM_SERIAL:
+		if (common->wcmProtocolLevel != 5)
+			return BadValue;
+		if (priv->serial != value)
+		{
+			priv->serial = value; 
+			xf86ReplaceIntOption(local->options, "Serial", priv->serial);
+		}
+		break;
 	    case XWACOM_PARAM_XYDEFAULT:
 		xf86WcmSetParam (local, XWACOM_PARAM_TOPX, 0);
 		xf86WcmSetParam (local, XWACOM_PARAM_TOPY, 0);
@@ -371,7 +386,7 @@ static int xf86WcmSetParam(LocalDevicePtr local, int param, int value)
 		}
 		break;
 	    case XWACOM_PARAM_CAPACITY:
-		if ((value < -1) || (value > 4)) 
+		if ((value < -1) || (value > 5)) 
 			return BadValue;
 		else if (common->wcmCapacity != value)
 		{
@@ -400,7 +415,7 @@ static int xf86WcmSetParam(LocalDevicePtr local, int param, int value)
 	    case XWACOM_PARAM_TWINVIEW:
 		if (priv->twinview != value)
 		{
-			if ((value > 2) || (value < 0) || screenInfo.numScreens != 1)
+			if ((value > TV_MAX) || (value < TV_NONE) || screenInfo.numScreens != 1)
 				return BadValue;
 			priv->twinview = value;
 
@@ -421,7 +436,7 @@ static int xf86WcmSetParam(LocalDevicePtr local, int param, int value)
 	    case XWACOM_PARAM_TVRESOLUTION0:
 	    case XWACOM_PARAM_TVRESOLUTION1:
 	    {
-		if (priv->twinview == TV_NONE)
+		if (priv->twinview <= TV_XINERAMA)
 			return -1;
 		else
 		{
@@ -471,6 +486,9 @@ static int xf86WcmSetParam(LocalDevicePtr local, int param, int value)
 				}
 			}
 		}
+
+		/* reset screen info */
+		xf86WcmChangeScreen(local, priv->screen_no);
 		break;
 	    }
 	    case XWACOM_PARAM_COREEVENT:
@@ -490,7 +508,7 @@ static int xf86WcmSetParam(LocalDevicePtr local, int param, int value)
 	   case XWACOM_PARAM_ROTATE:
 		if ((value < 0) || (value > 3)) return BadValue;
 		if (common->wcmRotate != value)
-			xf86WcmRotateScreen(local, value);
+			xf86WcmRotateTablet(local, value);
 		break;
 	   default:
 		DBG(10, priv->debugLevel, ErrorF("xf86WcmSetParam invalid param %d\n",param));
@@ -795,6 +813,8 @@ static int xf86WcmGetParam(LocalDevicePtr local, int param)
 	    case XWACOM_PARAM_CLICKFORCE:
 		return !common->wcmMaxZ ? 0 :
 			(int) (((common->wcmThreshold + 0.5) * 100) / common->wcmMaxZ);
+	    case XWACOM_PARAM_THRESHOLD:
+		return !common->wcmMaxZ ? 0 : common->wcmThreshold;
 	    case XWACOM_PARAM_XYDEFAULT:
 		return -1;
 	    case XWACOM_PARAM_MMT:
@@ -815,17 +835,19 @@ static int xf86WcmGetParam(LocalDevicePtr local, int param)
 		return priv->old_device_id;
 	    case XWACOM_PARAM_TOOLSERIAL:
 		return priv->old_serial;
+	    case XWACOM_PARAM_SERIAL:
+		return priv->serial;
 	    case XWACOM_PARAM_TWINVIEW:
 		return priv->twinview;
 	    case XWACOM_PARAM_TVRESOLUTION0:
 	    case XWACOM_PARAM_TVRESOLUTION1:
 	    {
-		if (priv->twinview == TV_NONE)
+		if (priv->twinview <= TV_XINERAMA)
 			return -1;
 		else
 		{
-			int sNum = param - XWACOM_PARAM_TVRESOLUTION0;
-			return priv->tvResolution[sNum++] | (priv->tvResolution[sNum] << 16);
+			int sNum = 2 * (param - XWACOM_PARAM_TVRESOLUTION0);
+			return priv->tvResolution[sNum] | (priv->tvResolution[sNum + 1] << 16);
 		}
 	    }
 	    case XWACOM_PARAM_SCREEN_NO:
@@ -972,6 +994,11 @@ static int xf86WcmGetDefaultParam(LocalDevicePtr local, int param)
 		return 0;
 	case XWACOM_PARAM_CLICKFORCE:
 		return 6;
+	case XWACOM_PARAM_THRESHOLD:
+		if (strstr(common->wcmModel->name, "Intuos4"))
+			return (common->wcmMaxZ * 3 / 25);
+		else
+			return (common->wcmMaxZ * 3 / 50);
 	case XWACOM_PARAM_MMT:
 		return 1;
 	case XWACOM_PARAM_TPCBUTTON:
